# -*- coding: utf-8 -*-
"""
Texterra Python Interface
=========================

This package provides a Python interface for Texterra API.
Texterra is a toolkit for natural language processing and knowledge base utilization,
developed by a team of researchers at the Ivannikov Institute for System Programming.
Currently, English and Russian languages are supported.

See https://texterra.ispras.ru for more information about Texterra project.
"""
from .api import API
from .syntaxtree import SyntaxTree

__version__ = '1.0.0'



# -*- coding: utf-8 -*-
annotationName = 'spelling-correction-token'


def params():
    return {'targetType': annotationName}


def process(document, rtype=None, api=None):
    """ Extracts spelling-corrected tokens in specified format from given texterra-annotated text. """
    corrected_tokens = []
    if annotationName in document['annotations']:
        if rtype == 'annotation':
            for token in document['annotations'][annotationName]:
                corrected_tokens.append((token['start'], token['end'], token['value']))

        elif rtype == 'token':
            for token in document['annotations'][annotationName]:
                corrected_tokens.append((document['text'][token['start']: token['end']], token['value']))

        else:  # rtype == 'full'
            for token in document['annotations'][annotationName]:
                corrected_tokens.append((token['start'], token['end'], document['text'][token['start']: token['end']],
                                         token['value']))
    return corrected_tokens



# -*- coding: utf-8 -*-
from __future__ import unicode_literals


class SyntaxTree(object):
    """
    Implements a sentence's dependency parse tree.

    Attributes:
        tokens (list(str)): the list of the sentence's tokens, including the root element 'ROOT'

        spans (list(tuple(int, int))): the list of the sentence's tokens' (start index, end index) spans,
            with a 'None' value at the start for the root element.

        heads (list(int)): the list of indexes of each token's head element, where '1' corresponds to the first
            token in the sentence, and '0' corresponds to the root element. The list starts with a 'None' value
            for the root element (the root has no head element).

        labels (list(str)): the list of each token's dependency type from its head. The list starts with a
            'None' value for the root element (the root has no head element).

        tree (dict): a dictionary containing the sentence's parse tree, with root elements as key and their child
            elements as value. Child elements that have their own children are also stored as dictionaries, where
            they serve as key and their children as value.

        to_string (str): a string representation of the sentence's dependency parse.
    """

    def __init__(self, annotated_text):
        """
        Receives a Texterra-annotated text and initializes the syntax tree.
        """
        self.tokens = ['ROOT']  # initially has only root element
        self.spans = [None]
        self.heads = [None]  # root has no head element
        self.labels = [None]  # root has no head element => no label

        span_to_index = {}  # maps token spans to indexes
        root_indexes = []  # to store indexes of root elements

        # get token spans and values from the Texterra-annotated document
        for i, an in enumerate(annotated_text['annotations']['syntax-relation']):
            span = (an['start'], an['end'])
            self.spans.append(span)
            span_to_index[span] = i + 1
            self.tokens.append(annotated_text['text'][an['start']: an['end']])

        # iterate over the document again to set heads and labels
        for i, an in enumerate(annotated_text['annotations']['syntax-relation']):
            if 'parent' in an['value']:
                self.heads.append(span_to_index[(an['value']['parent']['start'], an['value']['parent']['end'])])
                self.labels.append(an['value']['type'])
            else:
                self.heads.append(0)
                self.labels.append('ROOT')
                root_indexes.append(i + 1)

        # stores dependency structure of the sentence in dict, with
        # root elements as key and their child elements as value.
        # child elements that have their own children are stored as dicts
        # where they serve as key and their children as value.
        self.tree = {}
        self._visited = []  # stores elements visited during tree's building process
        self.to_string = ''

        # iterate over root elements and build their subtrees
        for root_index in root_indexes:
            # get the root's span
            root_span = self.spans[root_index]

            # indicate the root as visited
            self._visited.append(root_index)

            # build the roots subtree
            sub_tree, sub_tree_string = self._build_tree(root_index)
            sub_tree_key = (root_span[0], root_span[1], self.tokens[root_index], 'ROOT')
            self.tree[sub_tree_key] = sub_tree

            # attach the subtrees string to the sentence's parse string
            if len(root_indexes) > 0 and not sub_tree_string.startswith('('):
                format_string = '({0}) '
            else:
                format_string = '{0} '
            self.to_string += format_string.format(sub_tree_string)

    def get_labels(self):
        """ Returns the list of each token's dependency type from its head. """
        return self.labels[1:]

    def get_heads(self):
        """
        Returns the list of indexes of each token's head element.
        In the returned list, '1' corresponds to the first token in the sentence, and
        '0' corresponds to the root element.
        """
        return self.heads[1:]

    def _build_tree(self, index):
        """
        Traverses parse tree's elements and recursively adds children of element at given index to the tree.

        :param index: head element's index
        """

        children = []
        to_string = '({0}/{1}'.format(self.tokens[index], self.labels[index])

        for i in range(1, len(self.tokens)):

            if i not in self._visited and self.heads[i] == index:
                self._visited.append(i)
                child_tree = {}
                c, s = self._build_tree(i)
                child_tree[(self.spans[i][0], self.spans[i][1], self.tokens[i], self.labels[i])] = c
                children.append(child_tree)
                to_string += ' {0}'.format(s)

        if len(children) > 0:
            to_string += ')'
            return children, to_string
        else:
            return children, to_string[1:]

    def __str__(self):
        return self.to_string



# -*- coding: utf-8 -*-
annotationName = 'keyconcepts'


def params():
    return {'targetType': annotationName}


def process(document, rtype=None, api=None):
    """ Receives a texterra-annotated text and returns the list of kb-article links for that text's concepts. """
    result = []
    if 'keyconcepts' in document['annotations']:
        ids, kbnames = [], []
        weights = {}
        for entry in document['annotations'][annotationName]:
            for concept in entry['value']:
                concept_id = concept['concept']['id']
                kb_name = concept['concept']['kb-name']
                ids.append(concept_id)
                kbnames.append(kb_name)
                weights['{0}:{1}'.format(concept_id, kb_name)] = concept['weight']

        if len(ids) > 0:
            concepts = []
            atrs = api._get_attributes(ids, kbnames, ['url'])
            for at in atrs:
                concepts.append((weights[at], atrs[at]['url']))
            concepts.sort(key=lambda x: -x[0])
            result = [c[1] for c in concepts]

    return result



# -*- coding: utf-8 -*-
from .. import syntaxtree

annotationName = 'syntax-relation'


def params():
    return {'targetType': annotationName}


def process(document, rtype=None, api=None):
    if annotationName in document['annotations']:
        return syntaxtree.SyntaxTree(document)
    else:
        return None



# -*- coding: utf-8 -*-
"""
Tools for Natural Language Processing.
"""
import sys
import six
from os import getenv
from . import feature
from . import utils


class API(utils.HttpBase):
    """
    This class provides methods for natural language processing with Texterra REST via OpenAPI.
    """

    # default texterra path
    texterra_version = 'v1'
    texterra_url = 'https://api.ispras.ru/texterra/{0}/'.format(texterra_version)

    max_batch_size = 1000000

    def __init__(self, key=getenv('TEXTERRA_KEY', False), host=getenv('TEXTERRA_HOST', None)):
        """
        Provide an API key to use the default Texterra version (v1).
        For a different version of Texterra, specify a custom host.

        :type key: str
        :type host: str
        """
        if host is None:
            host = self.texterra_url

            if key is None or len(key) != 40:
                raise ValueError('Invalid API key. Please provide a proper API key.')

        super(API, self).__init__(key, host)

    # NLP methods

    def language_detection(self, texts):
        """
        Detects given texts' language.

        :param texts: the texts to be language detected
        :type texts: list(str) or str
        :return: yields given texts' ISO 639-1 language codes
        :rtype: str generator
        """
        return self._process_texts(texts, feature.languagedetection)

    def sentence_detection(self, texts, rtype='full', domain='', language=''):
        """
        Detects boundaries of sentences in given texts.

        :param texts: the texts to be sentence tokenized
        :type texts: list(str) or str
        :param rtype: sets return type (the default is 'full'):
                     - 'sentence': list(str), i.e. list of detected sentences
                     - 'annotation': list(tuple(int, int)), i.e. list of detected sentences' start, end indexes
                     - 'full': list(tuple(int, int, str))
        :type rtype: str
        :param domain: specifies texts' domain (auto-detect if not provided)
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields lists of tuples, where tuple contains a sentence's start, end indexes and its value
        :rtype: generator of list(tuple(int, int, str))
        """
        return self._process_texts(texts, feature.sentencedetection, rtype=rtype, domain=domain, language=language)

    def tokenization(self, texts, rtype='full', domain='', language=''):
        """
        Detects all tokens (minimal significant text parts) in given texts.

        :param texts: the texts to be tokenized.
        :type texts: list(str) or str
        :param rtype: sets return type (the default is 'full'):
                    - 'token': list(str), i.e. list of detected tokens
                    - 'annotation': list(tuple(int, int)), i.e. list of detected tokens' start, end indexes
                    - 'full': list(tuple(int, int, str))
        :type rtype: str
        :param domain: specifies texts' domain (auto-detect if not provided)
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields lists of tuples, where each tuple contains a token's start, end indexes and its value.
        :rtype: generator of list(tuple(int, int, str))
        """
        return self._process_texts(texts, feature.tokenization, rtype=rtype, domain=domain, language=language)

    def lemmatization(self, texts, rtype='full', domain='', language=''):
        """
        Detects each word's lemma in given texts.

        :param texts: the texts to lemmatize
        :type texts: list(str) or str
        :param rtype: sets return type (the default is 'full'):
                     - 'lemma': list(str), i.e. list of lemmas
                     - 'annotation': list(tuple(int, int, str)), i.e. list of lemmas’ start, end indexes and value
                     - 'full': list(tuple(int, int, str, str))
        :type rtype: str
        :param domain: specifies texts' domain (auto-detect if not provided)
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields lists of tuples, where each tuple contains a token’s start, end indexes, its value and lemma
        :rtype: generator of list(tuple(int, int, str, str))
        """
        return self._process_texts(texts, feature.lemmatization, rtype=rtype, domain=domain, language=language)

    def pos_tagging(self, texts, rtype='full', domain='', language=''):
        """
        Detects each token's part of speech tag in given texts.
        The method uses the Penn Treebank tagset for English, and SynTagRus tags for Russian.

        :param texts: the text to be pos-tagged
        :type texts: list(str) or str
        :param rtype: sets return type (the default is 'full'):
                     - 'token': list(tuple(str, str)), i.e. list of detected tokens and their tags
                     - 'annotation': list(tuple(int, int, str)), i.e. list of tokens’ start, end indexes and tag
                     - 'full': list(tuple(int, int, str, str))
        :type rtype: str
        :param domain: specifies texts' domain (auto-detect if not provided)
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields lists of tuples, where each tuple contains a token’s start, end indexes, value, and tag
        :rtype: generator of list(tuple(int, int, str, str))
        """
        return self._process_texts(texts, feature.postagging, rtype=rtype, domain=domain, language=language)

    def spelling_correction(self, texts, rtype='full', domain='', language=''):
        """
        Tries to correct spelling errors in given texts.

        :param texts: the text to be corrected
        :type texts: list(str) or str
        :param rtype: sets return type (the default is 'full'):
                     - 'token': list(tuple(str, str)), i.e. list of tokens and their corrections
                     - 'annotation': list(tuple(int, int, str)), i.e. list of tokens’ start, end indexes and correction
                     - 'full': list(tuple(int, int, str, str))
        :type rtype: str
        :param domain: specifies texts' domain (auto-detect if not provided)
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields lists of tuples, where each tuple contains a token’s start, end indexes,
                its value and correction
        :rtype: generator of list(tuple(int, int, str, str))
        """
        return self._process_texts(texts, feature.spellingcorrection, rtype=rtype, domain=domain, language=language)

    def named_entities(self, texts, rtype='full', domain='', language=''):
        """
        Finds all named entities occurrences in given texts.

        :param texts: the text to be analyzed
        :type texts: list(str) or str
        :param rtype: sets return type (the default is 'full'):
                     - 'entity': list(tuple(str, str)), i.e. list of named entities and their BBN categories
                     - 'full': list(tuple(int, int, str, str))
        :type rtype: str
        :param domain: specifies texts' domain (auto-detect if not provided)
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields lists of tuples, where each tuple contains a named entity’s start, end indexes, value, and
                 BBN category
        :rtype: generator of list(tuple(int, int, str, str))
        """
        return self._process_texts(texts, feature.namedentities, rtype=rtype, domain=domain, language=language)

    def disambiguation(self, texts, domain='', language=''):
        """
        Detects the most appropriate meanings (concepts) for terms occurred in a given text.

        :param texts: the text to be disambiguated
        :type texts: list(str) or str
        :param domain: specifies texts' domain (auto-detect if not provided)
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields lists of tuples. Each tuple contains a term’s start, end indexes, value, and the link to
                 the corresponding article in KB.
        :rtype: generator of list(tuple(int, int, str, str))
        """
        return self._process_texts(texts, feature.disambiguation, domain=domain, language=language)

    def key_concepts(self, texts, domain='', language=''):
        """
        Key concepts are the concepts providing short (conceptual) and informative text description.
        This service extracts the key concepts for given texts.

        :param texts: the text to be analyzed
        :type texts: list(str) or str
        :param domain: specifies texts' domain (auto-detect if not provided)
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields lists of links to concepts' corresponding KB articles
                sorted by concept weight in descending order
        :rtype: generator of list(str)
        """
        return self._process_texts(texts, feature.keyconcepts, domain=domain, language=language)

    def subjectivity_detection(self, texts, domain='', language=''):
        """
        Detects for each of the given texts if it is subjective or not.

        :param texts: the texts to be analyzed
        :type texts: list(str) or str
        :param domain: specifies texts' domain (auto-detect if not provided)
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields True for subjective texts, otherwise False.
        :rtype: bool generator
        """
        return self._process_texts(texts, feature.subjectivitydetection, domain=domain, language=language)

    def polarity_detection(self, texts, domain='', language=''):
        """
        Detects whether given texts have positive, negative, or no sentiment, with respect to domain.
        If domain isn't provided, domain detection is applied, this way method tries to achieve best results.
        If no domain is detected general domain algorithm is applied.
        This method only accepts texts that are not empty and contain at least one non-whitespace character.

        :param texts: the texts to be analyzed
        :type texts: list(str) or str
        :param domain: domain
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields tuples containing a text's polarity and domain.
        :rtype: generator of tuple(str, str)
        """
        return self._process_texts(texts, feature.polaritydetection, domain=domain, language=language)

    def syntax_detection(self, sentences, domain='', language=''):
        """
        This method detects syntax relations in given sentences. It uses the Penn Treebank labels for English,
        and SynTagRus labels for Russian.
        For sentences that are empty or contain only whitespace characters, None is returned.

        :param sentences: the sentences to be parsed
        :type sentences: list(str) or str
        :param domain: domain
        :type domain: str
        :param language: texts' language ISO 639-1 code
        :type language: str
        :return: yields a SyntaxTree instance for each sentence
        :rtype: generator of SyntaxTree
        """
        return self._process_texts(sentences, feature.syntaxdetection, domain=domain, language=language)

    # KBM methods

    def _wrap_concepts(self, concepts, kbnames):
        """ Utility wrapper for matrix parameters """
        if isinstance(concepts, list):
            if isinstance(kbnames, list):
                return ''.join(['id={0}:{1};'.format(concept, kb) for concept, kb in zip(concepts, kbnames)])
            else:
                return ''.join(['id={0}:{1};'.format(concept, kbnames) for concept in concepts])
        else:
            return 'id={0}:{1};'.format(concepts, kbnames)

    def _get_attributes(self, concepts, kbnames, atr_list=None):
        """
        Get attributes for concepts(list or single concept, each concept is {id}, {kbname} is separate parameter).
        Supported attributes:
            'coordinates' - GPS coordinates
            'definition' - brief concept definition
            'url' or 'url(<language>)' - URL to page with description of the given concept on the specified language
            '<language>' - language code, like: en, de, fr, ko, ru, ...
            'synonym' - different textual representations of the concept
            'title' - concept title
            'translation(<language>)' textual representation of the concept on the specified language
            '<language>' - language code (e.g. en, de, fr, ko, ru, ...)
            'type' - concept type
        """
        params = {'attribute': atr_list or []}
        return self.get('walker/{}'.format(self._wrap_concepts(concepts, kbnames)), params)

    # Helper methods

    def _batch_query(self, texts, params):
        """ Invokes custom batch request to Texterra. Returns json. """
        result = self.post('nlp', params, json=texts)
        return result

    def _process_texts(self, texts, module, rtype=None, domain='', language=''):
        for batch in self._get_batches(texts):
            if len(batch) != 0:
                params = module.params()

                if language != '':
                    params['language'] = language

                if domain != '':
                    params['domain'] = domain

                for document in self._batch_query(batch, params):
                    yield module.process(document, rtype, api=self)

    def _get_batches(self, texts):
        """ Reads texts from iterator and yields in 1MB-size batches."""
        if isinstance(texts, six.string_types):
            # texts contains only one text
            # check if it's of acceptable size and yield it
            self._check_text_size(texts)
            yield [{'text': texts}]

        else:
            # collect texts into batches of size 1 MB
            batch = []
            batch_size = 0
            for text in texts:

                # check that text's size does not exceed memory limit
                text_size = self._check_text_size(text)

                # check if batch size including new text exceeds memory limit
                if batch_size + text_size >= self.max_batch_size:

                    # yield collected texts
                    yield batch

                    # start collecting new batch
                    batch_size = text_size
                    batch = [{'text': text}]

                else:
                    batch_size += text_size
                    batch.append({'text': text})

            yield batch

    def _check_text_size(self, text):
        """ Checks that the text's size does not exceed memory limit """
        text_size = sys.getsizeof(text)
        if text_size >= self.max_batch_size:
            msg = 'Given text "{0}..." is too large, size exceeds the limit of {1} bytes.'
            raise ValueError(msg.format(text[:50], self.max_batch_size))

        return text_size



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import unittest
import texterra
import types
import six
from os import getenv
from dotenv import load_dotenv, find_dotenv
load_dotenv(find_dotenv())


# Texterra Tests

class CustomTexterraAPITest(unittest.TestCase):

    def setUp(self):
        TEXTERRA_CUSTOM_HOST = getenv("TEXTERRA_CUSTOM_HOST")
        TEXTERRA_CUSTOM_KEY = getenv("TEXTERRA_CUSTOM_KEY")
        self.custom_texterra = texterra.API(host=TEXTERRA_CUSTOM_HOST, key=TEXTERRA_CUSTOM_KEY)

    def test_custom_get_attributes(self):
        self.assertIsInstance(self.custom_texterra._get_attributes(12, 'enwiki'), dict)


class TexterraAPITest(unittest.TestCase):

    def setUp(self):
        TEXTERRA_KEY = getenv("TEXTERRA_KEY")
        self.texterra = texterra.API(key=TEXTERRA_KEY)

        self.en_text = 'Apple today updated iMac to bring numerous high-performance enhancements to the leading all-in-one desktop. iMac now features fourth-generation Intel Core processors, new graphics, and next-generation Wi-Fi. In addition, it now supports PCIe-based flash storage, making its Fusion Drive and all-flash storage options up to 50 percent faster than the previous generation'
        self.ru_text = 'Первые в этом году переговоры министра иностранных дел России Сергея Лаврова и госсекретаря США Джона Керри, длившиеся 1,5 часа, завершились в Мюнхене.'
        self.en_tweet = 'mentioning veterens care which Mccain has voted AGAINST - SUPER GOOOOD point Obama+1 #tweetdebate'
        self.ru_tweet = 'В мастерской готовят пушку и автомобили 1940-х годов, для участия в Параде Победы в Ново-Переделкино.'

    def test_bad_key(self):
        with self.assertRaises(ValueError):
            texterra.API('too short')

    def test_key_concepts(self):
        # test return type
        self.assertIsInstance(self.texterra.key_concepts([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.key_concepts([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.key_concepts([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.key_concepts([self.ru_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]
        concepts = []

        # test for a single text
        for text in texts:
            for entry in self.texterra.key_concepts(text):
                self.assertIsInstance(entry, list)
                entry_concepts = []
                for concept in entry:
                    self.assertIsInstance(concept, six.string_types)
                    entry_concepts.append(concept)
                concepts.append(entry_concepts)

        # test for known example
        self.assertEqual(concepts[0], ["http://en.m.wikipedia.org/wiki/Flash_memory"])
        self.assertEqual(concepts[1][0],
                         "http://ru.m.wikipedia.org/wiki/%D0%9C%D0%B8%D0%BD%D0%B8%D1%81%D1%82%D1%80_%D0%B8%D0%BD%D0%BE%D1%81%D1%82%D1%80%D0%B0%D0%BD%D0%BD%D1%8B%D1%85_%D0%B4%D0%B5%D0%BB")

        # test for text iterator
        result = self.texterra.key_concepts(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, list)
            for concept in entry:
                self.assertIsInstance(concept, six.string_types)

        # test memory limit on large text
        format_string = 'Given text "{0}..." is too large, size exceeds the limit of {1} bytes.'
        expected_msg = format_string.format(self.en_text[:50], self.texterra.max_batch_size)
        with self.assertRaises(ValueError) as cm:
            concepts = next(self.texterra.key_concepts(100000 * self.en_text))
        self.assertEqual(str(cm.exception), expected_msg)

    def test_disambiguation(self):
        # test return type
        self.assertIsInstance(self.texterra.disambiguation([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.disambiguation([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.disambiguation([self.ru_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.disambiguation([self.en_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.disambiguation(text):
                self.assertIsInstance(entry, list)

        # test text iterator
        result = self.texterra.disambiguation(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, list)
            for term in entry:
                self.assertIsInstance(term, tuple)
                self.assertEqual(len(term), 4)
                self.assertIsInstance(term[0], int)
                self.assertIsInstance(term[1], int)
                self.assertIsInstance(term[2], six.string_types)
                self.assertIsInstance(term[3], six.string_types)

        # test english
        en_test_text = "iMac now supports PCIe-based flash storage, making its Fusion Drive and all-flash storage options up to 50 percent faster than the previous generation."
        en_result = next(self.texterra.disambiguation([en_test_text]))
        en_result_expected = [(29, 42, "flash storage", "http://en.m.wikipedia.org/wiki/Flash_memory")]
        self.assertEqual(en_result, en_result_expected)

        # test russian
        ru_test_text = "Согласно официальному прогнозу Минэкономразвития, ВВП России упадет на 3%."
        ru_result = next(self.texterra.disambiguation([ru_test_text]))
        ru_result_expected = [
            (22, 30, "прогнозу", "http://ru.m.wikipedia.org/wiki/%D0%9F%D1%80%D0%BE%D0%B3%D0%BD%D0%BE%D0%B7"),
            (31, 48, "Минэкономразвития",
             "http://ru.m.wikipedia.org/wiki/%D0%9C%D0%B8%D0%BD%D0%B8%D1%81%D1%82%D0%B5%D1%80%D1%81%D1%82%D0%B2%D0%BE_%D1%8D%D0%BA%D0%BE%D0%BD%D0%BE%D0%BC%D0%B8%D1%87%D0%B5%D1%81%D0%BA%D0%BE%D0%B3%D0%BE_%D1%80%D0%B0%D0%B7%D0%B2%D0%B8%D1%82%D0%B8%D1%8F_%D0%A0%D0%BE%D1%81%D1%81%D0%B8%D0%B9%D1%81%D0%BA%D0%BE%D0%B9_%D0%A4%D0%B5%D0%B4%D0%B5%D1%80%D0%B0%D1%86%D0%B8%D0%B8"),
            (50, 60, "ВВП России",
             "http://ru.m.wikipedia.org/wiki/%D0%92%D0%92%D0%9F_%D0%A0%D0%BE%D1%81%D1%81%D0%B8%D0%B8")]
        self.assertEqual(len(ru_result), 3)
        for term in ru_result_expected:
            self.assertEqual(term in ru_result, True)

    def test_syntax_detection(self):
        # test return type
        self.assertIsInstance(self.texterra.syntax_detection([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.syntax_detection([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.syntax_detection([self.ru_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.syntax_detection([self.en_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.syntax_detection(text):
                self.assertIsInstance(entry, texterra.SyntaxTree)

        # test for text iterator
        result = self.texterra.syntax_detection(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, texterra.SyntaxTree)
            self.assertIsInstance(entry.to_string, six.string_types)
            self.assertEqual(len(entry.heads), len(entry.labels))
            self.assertEqual(0 in entry.heads, True)
            self.assertEqual('ROOT' in entry.labels, True)

        # test english
        en_test_sent = "Our kids should grow up in an America where opportunity is real."
        en_result = next(self.texterra.syntax_detection([en_test_sent]))
        self.assertIsInstance(en_result, texterra.SyntaxTree)
        self.assertEqual(en_result.get_labels(),
                         ["advmod", "nsubj", "dep", "ROOT", "advmod", "prep", "det", "pobj", "punct", "nsubj",
                          "cop", "rcmod", "dep"])
        self.assertEqual(en_result.to_string,
                         "(grow/ROOT Our/advmod kids/nsubj should/dep up/advmod (in/prep (America/pobj an/det (real/rcmod where/punct opportunity/nsubj is/cop ./dep)))) ")

        # test russian
        ru_test_sent = "Согласно официальному прогнозу Минэкономразвития, ВВП России упадет на 3%."
        ru_result = next(self.texterra.syntax_detection([ru_test_sent]))
        self.assertIsInstance(ru_result, texterra.SyntaxTree)
        self.assertEqual(ru_result.get_labels(),
                         ["обст", "опред", "предл", "квазиагент", "PUNCT", "предик", "квазиагент", "ROOT", "2-компл",
                          "предл", "PUNCT"])
        self.assertEqual(ru_result.to_string,
                         "(упадет/ROOT (Согласно/обст (прогнозу/предл официальному/опред (Минэкономразвития/квазиагент ,/PUNCT))) (ВВП/предик России/квазиагент) (на/2-компл (3%/предл ./PUNCT))) ")

    def test_language_detection(self):
        # test return type
        self.assertIsInstance(self.texterra.language_detection([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.language_detection([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.language_detection([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.language_detection([self.ru_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.language_detection(text):
                self.assertIsInstance(entry, six.string_types)

        # test for text iterator
        result = self.texterra.language_detection(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, six.string_types)

        # test english
        lang_list = list(self.texterra.language_detection(texts))
        self.assertEqual(lang_list[0], 'en')
        self.assertEqual(lang_list[2], 'en')

        # test russian
        self.assertEqual(lang_list[1], 'ru')
        self.assertEqual(lang_list[3], 'ru')

    def test_sentence_detection(self):
        # test return type
        self.assertIsInstance(self.texterra.sentence_detection([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.sentence_detection([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.sentence_detection([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.sentence_detection([self.ru_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.sentence_detection(text):
                self.assertIsInstance(entry, list)

        # test for text iterator
        result = self.texterra.sentence_detection(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, list)
            for sentence in entry:
                self.assertIsInstance(sentence, tuple)
                self.assertIsInstance(sentence[0], int)
                self.assertIsInstance(sentence[1], int)
                self.assertIsInstance(sentence[2], six.string_types)

        # test english
        en_test_text = "If you've never tried it, I think it's an interesting exercise to do without the Python semantics. It does make you appreciate what the language is providing."
        en_result = next(self.texterra.sentence_detection([en_test_text]))
        en_result_expected = [(0, 98,
                               "If you've never tried it, I think it's an interesting exercise to do without the Python semantics."),
                              (99, 158, "It does make you appreciate what the language is providing.")]
        self.assertEqual(en_result, en_result_expected)

        # test russian
        ru_test_text = "Выражение является полноправным оператором в Python. Состав, синтаксис, ассоциативность и приоритет операций достаточно привычны для языков программирования и призваны минимизировать употребление скобок."
        ru_result = next(self.texterra.sentence_detection([ru_test_text]))
        ru_result_expected = [(0, 52, "Выражение является полноправным оператором в Python."),
                              (53, 203,
                               "Состав, синтаксис, ассоциативность и приоритет операций достаточно привычны для языков программирования и призваны минимизировать употребление скобок.")]
        self.assertEqual(ru_result, ru_result_expected)

    def test_tokenization(self):
        # test return type
        self.assertIsInstance(self.texterra.tokenization([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.tokenization([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.tokenization([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.tokenization([self.ru_tweet]), types.GeneratorType)

        # test entry types
        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]
        for text in texts:
            for entry in self.texterra.tokenization([text]):
                self.assertIsInstance(entry, list)

        # test batch
        result = self.texterra.tokenization(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, list)

    def test_lemmatization(self):
        # test return type
        self.assertIsInstance(self.texterra.lemmatization([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.lemmatization([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.lemmatization([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.lemmatization([self.ru_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.lemmatization(text):
                self.assertIsInstance(entry, list)

        # test for text iterator
        result = self.texterra.lemmatization(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, list)
            for lemma in entry:
                self.assertIsInstance(lemma, tuple)
                self.assertIsInstance(lemma[0], int)
                self.assertIsInstance(lemma[1], int)
                self.assertIsInstance(lemma[2], six.string_types)
                self.assertIsInstance(lemma[3], six.string_types)

        en_test_sent = "Our kids should grow up in an America where opportunity is real."
        ru_test_sent = "Согласно официальному прогнозу Минэкономразвития, ВВП России упадет на 3%."
        result = self.texterra.lemmatization([en_test_sent, ru_test_sent])

        # test english
        en_result = next(result)
        en_result_expected = [(0, 3, "Our", "our"), (4, 8, "kids", "kid"), (9, 15, "should", "should"),
                              (16, 20, "grow", "grow"), (21, 23, "up", "up"), (24, 26, "in", "in"),
                              (27, 29, "an", "an"),
                              (30, 37, "America", "america"), (38, 43, "where", "where"),
                              (44, 55, "opportunity", "opportunity"),
                              (56, 58, "is", "be"), (59, 63, "real", "real"), (63, 64, ".", "")]
        self.assertEqual(en_result, en_result_expected)

        # test russian
        ru_result = next(result)
        ru_result_expected = [(0, 8, "Согласно", "согласно"), (9, 21, "официальному", "официальный"),
                              (22, 30, "прогнозу", "прогноз"), (31, 48, "Минэкономразвития", "минэкономразвития"),
                              (48, 49, ",", ""), (50, 53, "ВВП", "ввп"), (54, 60, "России", "россия"),
                              (61, 67, "упадет", "падать"), (68, 70, "на", "на"), (71, 73, "3%", "3"),
                              (73, 74, ".", "")]
        self.assertEqual(ru_result, ru_result_expected)

    def test_pos_tagging(self):
        # test return type
        self.assertIsInstance(self.texterra.pos_tagging([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.pos_tagging([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.pos_tagging([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.pos_tagging([self.ru_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.pos_tagging(text):
                self.assertIsInstance(entry, list)

        # test for text iterator
        result = self.texterra.pos_tagging(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, list)
            for pos_token in entry:
                self.assertIsInstance(pos_token, tuple)
                self.assertIsInstance(pos_token[0], int)
                self.assertIsInstance(pos_token[1], int)
                self.assertIsInstance(pos_token[2], six.string_types)
                self.assertIsInstance(pos_token[3], six.string_types)

        en_test_sent = "Our kids should grow up in an America where opportunity is real."
        ru_test_sent = "Согласно официальному прогнозу Минэкономразвития, ВВП России упадет на 3%."
        result = self.texterra.pos_tagging([en_test_sent, ru_test_sent])

        # test english
        en_result = next(result)
        en_result_expected = [(0, 3, "Our", "OTHER"), (4, 8, "kids", "NNS"), (9, 15, "should", "OTHER"),
                              (16, 20, "grow", "VB"), (21, 23, "up", "OTHER"), (24, 26, "in", "IN"),
                              (27, 29, "an", "DT"), (30, 37, "America", "NNP"), (38, 43, "where", "OTHER"),
                              (44, 55, "opportunity", "NN"), (56, 58, "is", "VBZ"), (59, 63, "real", "JJ"),
                              (63, 64, ".", "DOT")]
        self.assertEqual(en_result, en_result_expected)

        # test russian
        ru_result = next(result)
        ru_result_expected = [(0, 8, "Согласно", "PR"), (9, 21, "официальному", "A"), (22, 30, "прогнозу", "S"),
                              (31, 48, "Минэкономразвития", "S"), (48, 49, ",", "PUNCT"), (50, 53, "ВВП", "S"),
                              (54, 60, "России", "S"), (61, 67, "упадет", "V"), (68, 70, "на", "PR"),
                              (71, 73, "3%", "S"), (73, 74, ".", "PUNCT")]
        self.assertEqual(ru_result, ru_result_expected)

    def test_named_entities(self):
        # test return type
        self.assertIsInstance(self.texterra.named_entities([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.named_entities([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.named_entities([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.named_entities([self.ru_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.named_entities(text):
                self.assertIsInstance(entry, list)

        # test for text iterator
        result = self.texterra.named_entities(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, list)

        en_test_sent = "Our kids should grow up in an America where opportunity is real."
        ru_test_sent = "Согласно официальному прогнозу Минэкономразвития, ВВП России упадет на 3%."
        result = self.texterra.named_entities([en_test_sent, ru_test_sent])

        # test english
        en_result = next(result)
        en_result_expected = [(30, 37, "America", "LOCATION")]
        self.assertEqual(en_result, en_result_expected)

        # test russian
        ru_result = next(result)
        ru_result_expected = [(31, 48, "Минэкономразвития", "ORGANIZATION_POLITICAL"),
                              (54, 60, "России", "GPE_COUNTRY")]
        self.assertEqual(ru_result, ru_result_expected)

    def test_subjectivity_detection(self):
        # test return type
        self.assertIsInstance(self.texterra.subjectivity_detection([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.subjectivity_detection([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.subjectivity_detection([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.subjectivity_detection([self.ru_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.subjectivity_detection(text):
                self.assertIsInstance(entry, bool)

        # test for text iterator
        result = self.texterra.subjectivity_detection(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, bool)

        en_result = next(self.texterra.subjectivity_detection([self.en_tweet]))
        en_result_expected = True
        self.assertEqual(en_result, en_result_expected)

    def test_polarity_detection(self):
        # test return type
        self.assertIsInstance(self.texterra.polarity_detection([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.polarity_detection([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.polarity_detection([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.polarity_detection([self.ru_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.polarity_detection(text):
                self.assertIsInstance(entry, tuple)

        # test for text iterator
        result = self.texterra.polarity_detection(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, tuple)

        en_result = next(self.texterra.polarity_detection([self.en_tweet]))
        en_result_expected = "POSITIVE"
        self.assertEqual(en_result[0], en_result_expected)

    def test_spelling_correction(self):
        # test return type
        self.assertIsInstance(self.texterra.spelling_correction([self.en_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.spelling_correction([self.ru_text]), types.GeneratorType)
        self.assertIsInstance(self.texterra.spelling_correction([self.en_tweet]), types.GeneratorType)
        self.assertIsInstance(self.texterra.spelling_correction([self.ru_tweet]), types.GeneratorType)

        texts = [self.en_text, self.ru_text, self.en_tweet, self.ru_tweet]

        # test for a single text
        for text in texts:
            for entry in self.texterra.spelling_correction(text):
                self.assertIsInstance(entry, list)

        # test for text iterator
        result = self.texterra.spelling_correction(texts)
        self.assertIsInstance(result, types.GeneratorType)
        for entry in result:
            self.assertIsInstance(entry, list)
            for spelling_token in entry:
                self.assertIsInstance(spelling_token, tuple)
                self.assertIsInstance(spelling_token[0], int)
                self.assertIsInstance(spelling_token[1], int)
                self.assertIsInstance(spelling_token[2], six.string_types)
                self.assertIsInstance(spelling_token[3], six.string_types)

    def test_get_attributes(self):
        self.assertIsInstance(self.texterra._get_attributes(12, 'enwiki'), dict)
        self.assertIsInstance(self.texterra._get_attributes([12, 13137], 'enwiki'), dict)
        self.assertIsInstance(self.texterra._get_attributes(12, 'enwiki', ['url(en)', 'type']), dict)
        self.assertIsInstance(self.texterra._get_attributes([12, 13137], 'enwiki', ['url(en)', 'title']), dict)



annotationName = 'language'


def params():
    return {'targetType': annotationName}


def process(document, rtype=None, api=None):
    try:
        return document['annotations'][annotationName][0]['value']
    except KeyError:
        return ''



from . import (
    languagedetection,
    sentencedetection,
    tokenization,
    lemmatization,
    namedentities,
    postagging,
    disambiguation,
    keyconcepts,
    spellingcorrection,
    subjectivitydetection,
    polaritydetection,
    syntaxdetection
)



# -*- coding: utf-8 -*-
annotationName = 'sentence'


def params():
    return {'targetType': annotationName}


def process(document, rtype=None, api=None):
    """ Extracts sentences in specified format from given texterra-annotated text. """
    sents = []
    if annotationName in document['annotations']:
        if rtype == 'sentence':
            for sent in document['annotations'][annotationName]:
                sents.append(document['text'][sent['start']: sent['end']])

        elif rtype == 'annotation':
            for sent in document['annotations'][annotationName]:
                sents.append((sent['start'], sent['end']))

        else:  # if rtype == 'full':
            for sent in document['annotations'][annotationName]:
                sents.append((sent['start'], sent['end'], document['text'][sent['start']: sent['end']]))
    return sents



# -*- coding: utf-8 -*-
annotationName = 'subjectivity'


def params():
    return {'targetType': annotationName, 'tweet': 1}


def process(document, rtype=None, api=None):
    """ Detects subjectivity in given batch of texts. """
    """ Extracts subjectivity value from given texterra-annotated text. """
    return bool(document['annotations'])



# -*- coding: utf-8 -*-
annotationName = 'pos-token'


def params():
    return {'targetType': annotationName}


def process(document, rtype=None, api=None):
    """ Extracts tokens in specified format from given texterra-annotated text. """
    tokens = []
    if annotationName in document['annotations']:
        if rtype == 'token':
            for token in document['annotations'][annotationName]:
                tokens.append(document['text'][token['start']: token['end']])

        elif rtype == 'annotation':
            for token in document['annotations'][annotationName]:
                tokens.append((token['start'], token['end']))

        else:  # if rtype == 'full':
            for token in document['annotations'][annotationName]:
                tokens.append((token['start'], token['end'], document['text'][token['start']: token['end']]))

    return tokens



# -*- coding: utf-8 -*-
annotationName = 'named-entity'


def params():
    return {'targetType': annotationName}


def process(document, rtype=None, api=None):
    """ Extracts named entities in specified format from given texterra-annotated text."""
    entities = []
    if annotationName in document['annotations']:

        if rtype == 'entity':
            for token in document['annotations'][annotationName]:
                entities.append((document['text'][token['start']: token['end']], token['value']['tag']))

        else:  # rtype == 'full':
            for token in document['annotations'][annotationName]:
                entities.append((token['start'], token['end'], document['text'][token['start']: token['end']],
                                 token['value']['tag']))
    return entities



# -*- coding: utf-8 -*-
annotationName = 'pos-token'


def params():
    return {'targetType': annotationName}


def process(document, rtype=None, api=None):
    """ Extracts part-of-speech tagged tokens from given texterra-annotated text. """
    pos_tokens = []
    if annotationName in document['annotations']:
        if rtype == 'token':
            for token in document['annotations'][annotationName]:
                pos_tokens.append((document['text'][token['start']: token['end']], token['value']['tag']))

        elif rtype == 'annotation':
            for token in document['annotations'][annotationName]:
                pos_tokens.append((token['start'], token['end'], token['value']['tag']))

        else:  # if rtype == 'full':
            for token in document['annotations'][annotationName]:
                pos_tokens.append((token['start'], token['end'], document['text'][token['start']: token['end']],
                                   token['value']['tag']))
    return pos_tokens



annotationName = 'disambiguated-phrase'


def params():
    return {'targetType': annotationName}


def process(document, rtype=None, api=None):
    """ Receives a texterra-annotated annotated_text and returns its terms' list."""
    result = []
    if 'disambiguated-phrase' in document['annotations']:
        terms, ids, kbnames = [], [], []
        for term in document['annotations']['disambiguated-phrase']:
            term_id = term['value']['id']
            kb_name = term['value']['kb-name']
            ids.append(term_id)
            kbnames.append(kb_name)
            terms.append((term['start'], term['end'], document['text'][term['start']: term['end']],
                          '{0}:{1}'.format(term_id, kb_name)))

        if len(ids) > 0:
            atrs = api._get_attributes(ids, kbnames, ['url'])
            for term in terms:
                result.append((term[0], term[1], term[2], atrs[term[3]]['url']))

    return result




from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

# Read the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="texterra",
    version='1.0.1',
    description="API for natural language processing.",
    long_description=long_description,
    url="https://texterra.ispras.ru",
    license="Apache License, Version 2.0",
    keywords=['natural language processing', 'nlp', 'text processing', 'parsing', 'tagging', 'tokenizing', 'syntax',
              'language', 'lemmatization', 'named entity recognition', 'spelling correction', 'sentiment analysis',
              'disambiguation', 'key concepts detection'],
    classifiers=['Intended Audience :: Science/Research',
                 'Intended Audience :: Developers',
                 'License :: OSI Approved :: Apache Software License',
                 'Programming Language :: Python',
                 'Topic :: Software Development',
                 'Topic :: Scientific/Engineering',
                 'Topic :: Scientific/Engineering :: Artificial Intelligence',
                 'Topic :: Text Processing',
                 'Topic :: Text Processing :: Linguistic',
                 'Operating System :: OS Independent',
                 'Natural Language :: English',
                 'Natural Language :: Russian',
                 'Programming Language :: Python :: 2',
                 'Programming Language :: Python :: 3'
                 ],
    author="Tsolak Ghukasyan",
    author_email="texterra@ispras.ru",
    packages=find_packages(exclude=['tests']),
    install_requires=[
        'requests',
        'six'
    ]
)



# -*- coding: utf-8 -*-
annotationName = ['domain', 'polarity']


def params():
    return {'targetType': ['domain', 'polarity'], 'tweet': 1}


def process(document, rtype=None, api=None):
    """ Extracts polarity and domain values from given texterra-annotated text. """
    try:
        return document['annotations'][annotationName[1]][0]['value'], \
               document['annotations'][annotationName[0]][0]['value']
    except KeyError:
        if 'domain' in document['annotations']:
            return 'NEUTRAL', document['annotations'][annotationName[0]][0]['value']
        else:
            return 'NEUTRAL', 'general'



# -*- coding: utf-8 -*-
"""
Tools for HTTP JSON API
"""
import requests


class HttpBase(object):
    """ This class provides methods for sending GET and POST requests to JSON API. """

    def __init__(self, key, url):
        """
        :param url: base url for access
        """
        self.base_url = url
        self.apikey = key

    def get(self, path, params):
        """ Method for invoking API GET request """
        return self.custom_query(path, params)

    def post(self, path, params, json):
        """ Method for invoking API POST request """
        return self.custom_query(path, params, json)

    def custom_query(self, path, params, json=None):
        headers = {'Accept': 'application/json'}
        url = self.base_url + path
        if self.apikey:
            params['apikey'] = self.apikey

        if json is None:
            page = requests.get(url, params=params, headers=headers, timeout=60)
        else:
            page = requests.post(url, params=params, headers=headers, json=json, timeout=60)

        return self._process_page(page)

    def _process_page(self, page):
        page.raise_for_status()  # raises exception if not successful
        return page.json()
